@@ -54,23 +54,37 @@ module MergeRequests
         merge_request, merge_request.project, current_user, merge_request.assignee)
     end
 
-    def create_merge_request_pipeline(merge_request, user)
-      return unless Feature.enabled?(:ci_merge_request_pipeline,
-                                     merge_request.source_project,
-                                     default_enabled: true)
+    def create_pipeline_for(merge_request, user)
+      return unless can_create_pipeline_for?(merge_request)
 
+      create_detached_merge_request_pipeline(merge_request, user)
+    end
+
+    def create_detached_merge_request_pipeline(merge_request, user)
+      if can_use_merge_request_ref?(merge_request)
+        Ci::CreatePipelineService.new(merge_request.source_project, user,
+                                      ref: merge_request.ref_path)
+          .execute(:merge_request_event, merge_request: merge_request)
+      else
+        Ci::CreatePipelineService.new(merge_request.source_project, user,
+                                      ref: merge_request.source_branch)
+          .execute(:merge_request_event, merge_request: merge_request)
+      end
+    end
+
+    def can_create_pipeline_for?(merge_request)
       ##
       # UpdateMergeRequestsWorker could be retried by an exception.
-      # MR pipelines should not be recreated in such case.
-      return if merge_request.merge_request_pipeline_exists?
-      return if merge_request.has_no_commits?
+      # pipelines for merge request should not be recreated in such case.
+      return false if merge_request.merge_request_pipeline_exists?
+      return false if merge_request.has_no_commits?
 
-      Ci::CreatePipelineService
-        .new(merge_request.source_project, user, ref: merge_request.source_branch)
-        .execute(:merge_request_event,
-                 ignore_skip_ci: true,
-                 save_on_errors: false,
-                 merge_request: merge_request)
+      true
+    end
+
+    def can_use_merge_request_ref?(merge_request)
+      Feature.enabled?(:ci_use_merge_request_ref, project, default_enabled: true) &&
+        !merge_request.for_fork?
     end
 
     # Returns all origin and fork merge requests from `@project` satisfying passed arguments.
@@ -85,22 +99,11 @@ module MergeRequests
     # rubocop: enable CodeReuse/ActiveRecord
 
     def pipeline_merge_requests(pipeline)
-      merge_requests_for(pipeline.ref).each do |merge_request|
+      pipeline.all_merge_requests.opened.each do |merge_request|
         next unless pipeline == merge_request.head_pipeline
 
         yield merge_request
       end
     end
-
-    def commit_status_merge_requests(commit_status)
-      merge_requests_for(commit_status.ref).each do |merge_request|
-        pipeline = merge_request.head_pipeline
-
-        next unless pipeline
-        next unless pipeline.sha == commit_status.sha
-
-        yield merge_request
-      end
-    end
   end
 end
